import numpy as np
import sys
import os
import h5py
import re
import xarray as xr #cite as: Hoyer, S. & Hamman, J., (2017). xarray: N-D labeled Arrays and Datasets in Python. Journal of Open Research Software. 5(1), p.10. DOI: http://doi.org/10.5334/jors.148
import gc
import glob
import copy
import argparse
#from sklearn.neighbors import BallTree
import matplotlib.pyplot as plt
import netCDF4 as nv

import cartopy.crs as ccrs
import cartopy.feature as cf
from matplotlib import rcParams

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 20
rcParams['xtick.labelsize'] = 20
rcParams['ytick.labelsize'] = 20
rcParams['axes.labelsize'] = 28
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'


import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.colors as colors
from matplotlib.colors import LogNorm
from matplotlib import gridspec
from matplotlib import ticker
import matplotlib as mpl
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import NullFormatter
from collections import OrderedDict, Counter
from matplotlib.lines import Line2D
from matplotlib.patches import Patch, Rectangle, Ellipse
from scipy import stats
import matplotlib.patheffects as pe

import pandas as pd

class Grid():

  def __init__(self, path_to_directory="/home/lucie/FP_Anne_tests/FP11_BC1_numpf3/Output_eta", 
    fname = "grid_conc_19950118030000_nest.nc"):

    self.path = path_to_directory
    self.fname = fname

    self._read_file()

  def _read_file(self):
    tempfile = self.path + '/' + self.fname
    self._data = nv.Dataset(tempfile, "r", format="NETCDF4")
    print("netCDF4 file opened.")

  @property
  def wd(self):
    return self._data['WD_spec002'][0,0,:,:,:]

  @property
  def dataset(self):
    return self._data['spec001_mr']

  @property
  def receptordata(self):
    return self._data['receptor_conc001']

  @property
  def longitude(self):
    return np.array(self._data['longitude'][:])

  @property
  def latitude(self):
    return np.array(self._data['latitude'][:])

  @property
  def height(self):
    return np.array(self._data['height'][:])

  @property
  def receptor(self):
    return np.array(self._data['receptor'][:])

  @property
  def data(self):
    return self.dataset[0,0,:,:,:,:]

  def concentration(self,lat,lon,height=None):
    ilon=np.abs(self.longitude-lon).argmin()
    ilat=np.abs(self.latitude-lat).argmin()
    if height is None:
      return np.sum(np.array(self.data[:,0:1,ilat,ilon]),axis=1)
    else:
      iheight=np.abs(self.height-height).argmin()
      dh1=np.abs(self.height[iheight]-height)/(self.height[iheight+1]-self.height[iheight])
      return ((1.-dh1)*(self.data[:,iheight,ilat,ilon])+
        dh1*np.array(self.data[:,iheight+1,ilat,ilon]))

def haversine(lo1,la1,lo2,la2):
  # convert decimal degrees to radians 
  # lo1 = np.radians(lon1)
  # lo2 = np.radians(lon2)
  # la1 = np.radians(lat1)
  # la2 = np.radians(lat2)

  # haversine formula 
  dlon = lo2 - lo1 
  dlat = la2 - la1 
  a = np.sin(dlat/2)**2 + np.cos(la1) * np.cos(la2) * np.sin(dlon/2)**2
  c = 2. * np.arcsin(np.sqrt(a)) 
  r = 6371229. # Radius of earth in m.
  return c * r

def plot_concentration_map(ldir=(["/home/lucie/FP_Anne_tests/FP10",
  "/home/lucie/FP_Anne_tests/FP11_BC2",
  "/home/lucie/FP_Anne_tests/FP11_BC2_numpf3"]),fname='/home/lucie/FP11/wetdepo'):

  grid={}

  names=(["FP10",
    "FP11",
    "FP11+npf3"])
  for ld in ldir:
    grid[ld]=Grid(path_to_directory=ld)
    print(grid[ld].wd.shape)

  lon=grid[ldir[1]].longitude
  lat=grid[ldir[1]].latitude
  lonnew,latnew = np.meshgrid( lon, lat)

  lon2d, lat2d = np.meshgrid((lon[1:]+lon[:-1])/2., 
    (lat[1:]+lat[:-1])/2)
  grid_area = np.zeros_like(lon2d)
  grid_xrad = np.deg2rad(lon)
  grid_yrad = np.deg2rad(lat)+0.5*np.pi
  # for i in range(len(grid_xrad)-1):
  #   for j in range(len(grid_yrad)-1):
  #     grid_area[j, i] = (grid_xrad[i+1]-grid_xrad[i])*(np.cos(grid_yrad[j])-np.cos(grid_yrad[j+1]))*6371229.

  for i in range(len(grid_xrad)-1):
    for j in range(len(grid_yrad)-1):
      grid_area[j,i]=(haversine(grid_xrad[i],grid_yrad[j],grid_xrad[i+1],grid_yrad[j])*
        haversine(grid_xrad[i],grid_yrad[j],grid_xrad[i],grid_yrad[j+1]))

  levels=np.logspace(np.log10(0.00001),1,100)
  #cmap=copy.copy(plt.cm.PuBu)
  cmap=copy.copy(plt.cm.gist_earth_r)
  #cmap=copy.copy(plt.cm.viridis_r)

  tags=np.array(['(a)', '(b)', '(c)'])
  jj=0
  for j in [-1]: #range(totlen):
    fig,axes=plt.subplots(figsize=[25,7],nrows=1,ncols=3,subplot_kw={'projection':ccrs.PlateCarree(central_longitude=0)})
    for ax in axes:
      ax.add_feature(cf.COASTLINE)
      ax.add_feature(cf.BORDERS)
      #ax.add_feature(cf.RIVERS)
      ax.add_feature(cf.STATES,linewidth=0.1)
      ax.set_extent( [-2.5,31.537,36.,59] )
      cbt=ax.contour(lonnew, latnew, grid[ldir[jj]].wd[j,:,:],cmap=cmap,levels=levels,
          norm=LogNorm(),extend='both',zorder=0,linewidth=0.1)
      cbt=ax.contourf(lonnew, latnew, grid[ldir[jj]].wd[j,:,:],cmap=cmap,levels=levels,
          norm=LogNorm(),extend='both',zorder=0)
      ax.scatter(0.5678,41.201,facecolor="None", edgecolor='r',marker="*",s=100,)
      print(np.sum(grid[ldir[jj]].wd[j,:-1,:-1]*grid_area))
      ax.set_title(names[jj]+', Total: %1.2f g' %(np.sum(grid[ldir[jj]].wd[j,:-1,:-1]*grid_area*1e-12*1e3))+
        '\nMax: %1.2f$\\times 10^{-12}$ kg m$^{-2}$' %(np.max(grid[ldir[jj]].wd[j,:,:])),fontsize=28)
      ax.set_xticks([0, 15, 30], crs=ccrs.PlateCarree())
      ax.set_xticklabels(labels=['0$^{\\circ}$ E','15$^{\\circ}$ E','30$^{\\circ}$ E'],fontsize=28)
      ax.gridlines()
      ax.set_xlabel('Longitude',fontsize=28)
      ax.text(0.01,.99,tags[jj],horizontalalignment='left',verticalalignment='top',transform=ax.transAxes,color='k',fontsize=28,
        bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))
      jj=jj+1

  axes[0].set_yticks([40, 50, 60], crs=ccrs.PlateCarree())
  axes[0].set_yticklabels(labels=['40$^{\\circ}$ N','50$^{\\circ}$ N','60$^{\\circ}$ N'],fontsize=28)
  axes[0].set_ylabel('Latitude',fontsize=28)
  # plt.gca().gridlines()

  plt.tight_layout()
  fig.subplots_adjust(right=0.93)
  cbt_ax=fig.add_axes((0.94,0.08, 0.01,0.85))
  cb=plt.colorbar(cbt,cax=cbt_ax,orientation='vertical',ticks=[0.0001,0.0003,0.001,0.003,0.01,0.03,0.1,0.3,1,10],fraction=0.1)

  cb.cmap.set_under('white')
  cb.cmap.set_over(cmap(1.))
  cb.set_label('Deposition [$10^{-12}$ kg m$^{-2}$]',fontsize=28)

  #plt.savefig('/home/lucie/CAPTEX/plots/release1/etalog/'+fbasename+'%i.png' %j)
  # plt.tight_layout()
  plt.savefig(fname+'.png')
  plt.savefig(fname+'.pdf')
  plt.close()

